'''
Created by Han Xu
email:736946693@qq.com
'''
import os
import re
import requests
from urllib.parse import urlparse

class normalizer:
    def __init__(self,current_dir):
        self.cur_dir=current_dir

    def __call__(self, *args, **kwargs):
        self.process(self.cur_dir)


    def process_markdown_file(self,current_dir, md_file_name):
        os.chdir(current_dir)

        # 图片素材文件夹
        filename_noExt=os.path.splitext(md_file_name)[0]
        img_dir = 'assets'+filename_noExt if len('assets'+filename_noExt)<=200 else 'assets'+filename_noExt[:200]

        # 创建目录
        if os.path.exists(img_dir)==False:
            os.makedirs(img_dir)

        with open(md_file_name, 'r', encoding='utf-8') as f:
            md_content = f.read()

        img_pattern = re.compile('!\[.*?\]\((http.*?)\)', re.IGNORECASE)
        img_matches = img_pattern.findall(md_content)

        for img_url in img_matches:
            img_name = os.path.basename(urlparse(img_url).path)
            img_path = os.path.join(img_dir, img_name)

            img_dir_safe = img_dir.replace(' ', '%20')

            response = requests.get(img_url)
            with open(img_path, 'wb') as img_file:
                img_file.write(response.content)

            md_content = md_content.replace(img_url, f'{img_dir_safe}/{img_name}')

        with open(md_file_name, 'w', encoding='utf-8') as f:
            f.write(md_content)

        print(f'处理完成：{md_file_name}')

    def process(self,directory):

        if os.path.isdir(directory)==True:
            os.chdir(directory)
            obj_list=os.listdir(directory)


            for obj in obj_list:

                if obj==".git":continue

                elif obj.endswith('.md')==True:
                    self.process_markdown_file(directory,obj)
                elif os.path.isdir(os.path.join(directory,obj))==True:
                    self.process(os.path.join(directory, obj))

        elif os.path.isdir(directory)==False and directory.endswith('.md'):
            path_name=os.path.dirname(directory)
            file_name = os.path.basename(directory)
            self.process_markdown_file(path_name,"")


if __name__ == '__main__':
    # replace the directory you want to rename
    target_directory = r"D:\Desktop\demo"
    normalizer(current_dir=target_directory)()




# Created by Han Xu
# email:736946693@qq.com
import requests
import urllib.request
import urllib.parse
import os
from tqdm import tqdm

class Spider_baidu_image():
    def __init__(self):
        """
        @:brief
        @:return
        """
        self.path=input("type in the path where you want to reserve the images:")
        self.url = 'http://image.baidu.com/search/acjson?'
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36 Edg/112.0.1722.48'}
        self.keyword = input("type in the keywords used to search in Baidu:")
        self.paginator = int(input("Type in the number of pages you want.Each page has 30 images:"))

    def get_urls(self):
        """
        @:brief Get the URLs that you need to visit.
        @:return return a list of the URLs
        """
        keyword = urllib.parse.quote(self.keyword)
        params = []
        for i in range(1, self.paginator + 1):
            params.append(
                'tn=resultjson_com&ipn=rj&ct=201326592&is=&fp=result&queryWord={}&cl=2&lm=-1&ie=utf-8&oe=utf-8&adpicid=&st=-1&z=&ic=&hd=1&latest=0&copyright=0&word={}&s=&se=&tab=&width=&height=&face=0&istype=2&qc=&nc=1&fr=&expermode=&force=&cg=star&pn={}&rn=30&gsm=78&1557125391211='.format(
                    keyword, keyword, 30 * i))
        urls = []
        for i in params:
            urls.append(self.url + i)
        return urls


    def get_path(self):
        """
        @:brief Get the path where you want to reserve the images.
        @:return
        """
        dirname="./"+self.path
        dirname_origin = dirname
        int_index = 0
        while(True):
            IsExist = os.path.exists(dirname)
            if (IsExist==False):
                os.mkdir(dirname)
                IsCreate=True
                break
            else:
                int_index+=1
                dirname=dirname_origin+"({})".format(int_index)

        return dirname+"/"

    def get_image_url(self, urls):
        """
        @:brief Get the URLs of images.
        @:return a list of URLs of images
        """
        image_url = []
        for url in urls:
            json_data = requests.get(url, headers=self.headers).json()
            json_data = json_data.get('data')
            for i in json_data:
                if i:
                    image_url.append(i.get('thumbURL'))
        return image_url

    def get_image(self,image_url):
        """
        @:brief download the images into the path you set just
        @:return
        """
        m = 1
        for img_url in image_url:
            #定义一个flag用于判断下载图片是否异常
            flag=True
            try:
                #urlretrieve() 方法直接将远程数据下载到本地
                print("第{}张图片的URL是{}".format(m,img_url))
                print("保存于{}".format(os.getcwd()+self.path[1:]))

                image_path=self.path + str(m) + '.jpg'
                with tqdm(unit="B", unit_scale=True, unit_divisor=1024, miniters=1, mininterval=1,
                          desc=img_url.split("/")[-1]) as t:
                    urllib.request.urlretrieve(url=img_url, filename=image_path, reporthook=lambda a, b, c: t.update(b))

            except BaseException as error:
                    flag=False
                    print(error)
            if(flag):
                #下载完成提示
                print('**********第'+str(m)+'张图片下载完成********')
                #每下载完后一张,m累加一次
                m = m + 1
        print('下载完成!')
        return

    def __call__(self, *args, **kwargs):
        """
        @brief the constrcution of the class
        @:return
        """
        self.path=self.get_path()
        urls = self.get_urls()
        image_url = self.get_image_url(urls)
        self.get_image(image_url)
        return

if __name__ == "__main__":
    ex=Spider_baidu_image()
    ex()


import ToolBoxByHanXu.normalize_md as normalize_md
import ToolBoxByHanXu.rename_files
import urllib.request
from tqdm import tqdm
if __name__ == '__main__':

    # replace the directory you want to rename
    target_directory = r"/home/han/Desktop/MD_notes（复件）/online"
    normalize_md.normalizer(target_directory)()

    img_url = "https://gitee.com/UnderTurrets/python-template/raw/master/res/READMEimgRes/7ac23192b1904eb790272d8462cec5b8.png"
    image_path = r"7ac23192b1904eb790272d8462cec5b8.png"

    with tqdm(unit="B", unit_scale=True, unit_divisor=1024, miniters=1, mininterval=1,
              desc=img_url.split("/")[-1]) as t:
        urllib.request.urlretrieve(url=img_url, filename=image_path, reporthook=lambda a, b, c: t.update(b))



# Created by Han Xu
# email:736946693@qq.com
import requests
import urllib.parse
import os
import re
import urllib.request
import urllib.error
from tqdm import tqdm




opener = urllib.request.build_opener()
opener.addheaders = [('User-Agent','Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36 Edg/112.0.1722.48'),
                     ("referer", "https://www.pixiv.net/"),]
urllib.request.install_opener(opener)

class pixiv_spider():
    def __init__(self):
        """
        @:brief
        @:return
        """
        search_target_dic = {
            "1": "Rank",
            "2": "Keyword"
        }
        print(search_target_dic)
        self.searchTarget = ""
        while (True):
            self.searchTarget = input("Type in the target you want to search.\n")
            if (self.searchTarget in search_target_dic):
                break
            else:
                print("It seems that you have typed in a wrong answer.Please try again.")

        search_mode_dic = {
            "1": "Simple:Recommended and Fast because the downloaded images are vague.You can view the simple image and select what you love",
            "2": "Specific:Not Recommended and Slow because the downloaded images are clear"}
        print(search_mode_dic)
        self.searchMode = ""
        while (True):
            self.searchMode = input("Type in the requirement of download-mode.\n")
            if (self.searchMode in search_mode_dic):
                break
            else:
                print("It seems that you have typed in a wrong answer.Please try again.")

        self.path = input("type in the path where you want to reserve the images:\n")
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36 Edg/112.0.1722.48',
            "referer": "https://www.pixiv.net/",
        }

        self.maximages = int(input("Type in the number of images you want:\n"))


        if (self.searchTarget == "1"):
            DateMode_dict = {"1": "daily", "2": "weekly", "3": "monthly", "4": "rookie", "5": "daily_ai", "6": "male",
                             "7": "female"}
            print(DateMode_dict)
            DateMode = ""
            while (True):
                DateMode_key = input("type in the searchMode you want\n")
                if (DateMode_key in DateMode_dict):
                    DateMode = DateMode_dict[DateMode_key]
                    break
                else:
                    print("It seems that you have typed in a wrong answer.Please try again.")

            self.paginator = int(self.maximages / 50) + 1
            self.date = input("Type in the date you want to search.Follow the format like this:20230423\n")
            self.url_front = "https://www.pixiv.net/ranking.php?mode={}&date={}&format=json&p=".format(DateMode,
                                                                                                       self.date)

        elif (self.searchTarget == "2"):
            heat_dic = {"0":"0","1": "500", "2": "1000", "3": "5000", "4": "10000", }
            print(heat_dic)
            heat_require = ""
            while (True):
                DateMode_key = input("Type in the requirement of heat.Larger the number is,Less the results are.\n")
                if (DateMode_key in heat_dic):
                    heat_require = heat_dic[DateMode_key]
                    break
                else:
                    print("It seems that you have typed in a wrong answer.Please try again.")

            self.url_front = "https://www.pixiv.net/ajax/search/artworks/"
            self.paginator = int(self.maximages / 60) + 1

            self.keyword = input("type in the keywords used to search in pixiv:\n")
            while(True):
                if (heat_require=="0"):
                    break
                else:
                    self.keyword = self.keyword + " " + heat_require + "users入り"
                    break

    def get_urls(self):
        """
        @:brief Get the URLs that you need to visit.
        @:return return a list of the URLs
        """
        urls = []
        if (self.searchTarget=="1"):
            for i in range(1, self.paginator + 1):
                urls.append(self.url_front + str(i))
            return urls
        elif(self.searchTarget=="2"):
            keyword = urllib.parse.quote(self.keyword)
            params = []
            for i in range(1, self.paginator + 1):
                params.append(
                    "{}?word={}&order=date_d&mode=all&p={}&s_mode=s_tag&type=all&lang=zh&version=c4032a21df2a82afb72b3fd753afb10a037befb5".format(
                        keyword,
                        keyword,
                        i))
            for i in params:
                urls.append(self.url_front + i)
        return urls

    def get_path(self):
        """
        @:brief Get the path where you want to reserve the images.
        @:return
        """
        dirname = os.getcwd() + "/" + self.path
        dirname_origin = dirname
        int_index = 0
        while (True):
            IsExist = os.path.exists(dirname)
            if (IsExist == False):
                os.mkdir(dirname)
                IsCreate = True
                break
            else:
                int_index += 1
                dirname = dirname_origin + "({})".format(int_index)

        return dirname + "/"

    def get_images_urls(self, urls):
        """
        @:brief Get the URLs of images.
        @:return a list of URLs of images
        """
        images_urls = []
        for each_url in urls:
            print("The url of keyword:{}".format(each_url))
            response = requests.get(each_url, headers=self.headers)
            if (response.status_code != 403) and (response.status_code != 404):
                url_txt = response.text
                pattern_string = "\"url\":\"https:[^\"]+jpg"
                image_list = re.findall(string=url_txt, pattern=re.compile(pattern_string))

                for i in image_list:
                    if i and len(images_urls) < self.maximages:
                        i = i[i.rfind("img"):i.rfind("_p0")]
                        i = i.replace("\\", "")
                        if (self.searchMode == "1"):
                            images_urls.append("https://i.pximg.net/img-master/" + i + "_p0_master1200.jpg")

                        elif (self.searchMode == "2"):
                            images_urls.append("https://i.pximg.net/img-original/" + i + "_p0.jpg")
            else:
                print("the invalid url:{}!".format(each_url))
                continue
        return images_urls

    def get_images(self, images_urls):
        """
        @:brief download the images into the path you set just
        @:return
        """
        m = 1
        for img_url in images_urls:
            image_path=""
            try:
                image_path = self.path + str(m) + img_url[img_url.rfind("."):]

                with tqdm(unit="B", unit_scale=True, unit_divisor=1024, miniters=1, mininterval=1,
                          desc=img_url.split("/")[-1]) as t:
                    urllib.request.urlretrieve(url=img_url, filename=image_path, reporthook=lambda a, b, c: t.update(b))

                print("\n第{}张图片的URL是:\n{}".format(m, img_url))
                print('**********第' + str(m) + '张图片下载完成********')
                print("保存于:\n{}".format(image_path))
                continue

            except urllib.error.HTTPError:
                if (img_url[-3:] == "jpg"):
                    stable_url_png = img_url[:-3] + "png"
                    del img_url
                    img_url = stable_url_png

                elif (img_url[-3:] == "png"):
                    stable_url_jpg = img_url[:-3] + "jpg"
                    del img_url
                    img_url = stable_url_jpg

                image_path = self.path + str(m) + img_url[img_url.rfind("."):]

                with tqdm(unit="B", unit_scale=True, unit_divisor=1024, miniters=1, mininterval=1,
                          desc=img_url.split("/")[-1]) as t:
                    urllib.request.urlretrieve(url=img_url, filename=image_path, reporthook=lambda a, b, c: t.update(b))

                print("\n第{}张图片的URL是:\n{}".format(m, img_url))
                print('**********第' + str(m) + '张图片下载完成********')
                print("保存于:\n{}".format(image_path))
                continue

            except BaseException as error:
                print(error)
                print("There are some problems with the url:{}".format(img_url))
                continue

            finally:
                m+=1

        print('下载完成!')
        return

    def __call__(self, *args, **kwargs):
        """
        @brief the constrcution of the class
        @:return
        """
        self.path = self.get_path()
        urls = self.get_urls()
        image_urls = self.get_images_urls(urls=urls)
        self.get_images(image_urls)
        return



"""
Created by Han Xu
email:736946693@qq.com
"""
import os

class renamer:
    def __init__(self,current_dir):
        self.cur_dir=current_dir

    def __call__(self, new_name, target_type="*", order_key=None):
        self.rename_byType(new_name,target_type,order_key)

    def rename_byType(self, new_name, target_type="*", order_key=None):
        # 切换到指定目录
        os.chdir(self.cur_dir)

        # 获取目录下的所有文件
        files = os.listdir(self.cur_dir)
        # 跳过一些系统文件
        while (files.count(r"desktop.ini") != 0):
            files.remove(r"desktop.ini")
        # 排序
        files = sorted(files, key=order_key)

        index = 1
        # 遍历所有文件并重命名
        for file in files:

            if os.path.isdir(file) == False and file.rfind(".") != -1:

                # 获取文件后缀名
                file_type = file[file.rfind(".") + 1:]

                if target_type == "*" or target_type == file_type:
                    target_name = new_name + f"{index}." + file_type
                    os.rename(file, target_name)
                    print(f"{self.cur_dir}"
                          f"{file}-------->{target_name}", sep="\n")

                index = index + 1

        return


if __name__ == "__main__":
    ex=renamer(r"D:\Desktop\C & C++")
    ex(new_name="demo",target_type="jpg",order_key=lambda x: int(x[ x.rfind(".")-1 : x.rfind(".")+1] ) )


